#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

__version__ = '0.0.9'

import re
import os
import random
import time
import string
import socket
import sys
import traceback
import StringIO
import eulogger
import types
import operator

from functools import wraps


class TimeoutFunctionException(Exception): 
    """Exception to raise on a timeout""" 
    pass 


class Eutester(object):
    def __init__(self, credpath=None):
        """This class is intended to setup boto connections for the various services that the *ops classes will use.
        :param credpath: Path to a valid eucarc file.
        :param aws_access_key_id: Used in conjuction with aws_secret_access_key allows for creation of connections without needing a credpath.
        :param aws_secret_access_key: Used in conjuction with aws_access_key_id allows for creation of connections without needing a credpath.
        :rtype: :class:`eutester.Eutester` or ``None``
        :returns: A Eutester object with all connections that were able to be created. Currently EC2, S3, IAM, and STS.
        """
        ### Default values for configuration
        self.credpath = credpath
        
        ### Eutester logs
        self.logger = eulogger.Eulogger(identifier="EUTESTER")
        self.debug = self.logger.log.debug
        self.critical = self.logger.log.critical
        self.info = self.logger.log.info
        
        ### LOGS to keep for printing later
        self.fail_log = []
        self.running_log = self.logger.log

        ### Pull the access and secret keys from the eucarc or use the ones provided to the constructor
        if self.credpath is not None:
            self.debug("Extracting keys from " + self.credpath)         
            self.aws_access_key_id = self.get_access_key()
            self.aws_secret_access_key = self.get_secret_key()
            self.account_id = self.get_account_id()
            self.user_id = self.get_user_id()

    def get_access_key(self):
        if not self.aws_access_key_id:     
            """Parse the eucarc for the EC2_ACCESS_KEY"""
            self.aws_access_key_id = self.parse_eucarc("EC2_ACCESS_KEY")  
        return self.aws_access_key_id 
    
    def get_secret_key(self):
        if not self.aws_secret_access_key: 
            """Parse the eucarc for the EC2_SECRET_KEY"""
            self.aws_secret_access_key = self.parse_eucarc("EC2_SECRET_KEY")
        return self.aws_secret_access_key
    
    def get_account_id(self):
        if not self.account_id:
            """Parse the eucarc for the EC2_ACCOUNT_NUMBER"""
            self.account_id = self.parse_eucarc("EC2_ACCOUNT_NUMBER")
        return self.account_id
    
    def get_user_id(self):
        if not self.user_id:
            self.user_id = self.parse_eucarc("EC2_USER_ID")
        """Parse the eucarc for the EC2_ACCOUNT_NUMBER"""
        return self.user_id 

    def get_port(self):
        """Parse the eucarc for the EC2_ACCOUNT_NUMBER"""
        ec2_url = self.parse_eucarc("EC2_URL")
        return ec2_url.split(':')[1].split("/")[0]

    def parse_eucarc(self, field):
        with open( self.credpath + "/eucarc") as eucarc:
            for line in eucarc.readlines():
                if re.search(field, line):
                    return line.split("=")[1].strip().strip("'")
            raise Exception("Unable to find " +  field + " id in eucarc")
    
    def handle_timeout(self, signum, frame): 
        raise TimeoutFunctionException()

    def local(self, cmd):
        """
        Run a command on the localhost
        :param cmd: str representing the command to be run
        :return: :raise: CalledProcessError on non-zero return code
        """
        import subprocess
        args = cmd.split()
        process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=4096)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            error = subprocess.CalledProcessError(retcode, cmd)
            error.output = output
            raise error
        return output.split("\n")

    def found(self, command, regex):
        """ Returns a Boolean of whether the result of the command contains the regex
        """
        result = self.local(command)
        for line in result:
            found = re.search(regex,line)
            if found:
                return True
        return False
    
    def ping(self, address, poll_count = 10):
        """
        Ping an IP and poll_count times (Default = 10)
        address      Hostname to ping
        poll_count   The amount of times to try to ping the hostname iwth 2 second gaps in between
        """
        if re.search("0.0.0.0", address): 
            self.critical("Address is all 0s and will not be able to ping it") 
            return False
        self.debug("Attempting to ping " + address)
        while poll_count > 0:
            poll_count -= 1
            try:
                self.local("ping -c 1 " + address)
                self.debug("Was able to ping address")
                return True
            except:
                pass
            self.debug("Ping unsuccessful retrying in 2 seconds " + str(poll_count) + " more times")
            self.sleep(2)
        self.critical("Was unable to ping address")
        return False

    
    def scan_port_range(self, ip, start, stop, timeout=1, tcp=True):
        '''
        Attempts to connect to ports, returns list of ports which accepted a connection
        '''
        ret = []
        for x in xrange(start,stop+1):
            try:
                sys.stdout.write("\r\x1b[K"+str('scanning:'+str(x)))
                sys.stdout.flush()
                self.test_port_status(ip, x, timeout=timeout,tcp=tcp, verbose=False)
                ret.append(x)
            except socket.error, se:
                pass
        return ret
    
    def test_port_status(self, ip, port, timeout=5, tcp=True, verbose=True):
        '''
        Attempts to connect to tcp port at ip:port within timeout seconds
        '''
        if verbose:
            debug = self.debug
        else:
            debug = lambda msg: None
        debug('test_port_status, ip:'+str(ip)+', port:'+str(port)+', TCP:'+str(tcp))
        if tcp:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        else:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,socket.IPPROTO_UDP)
        s.settimeout(timeout)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            if tcp:
                s.connect((ip, port))
            else:
                s.sendto("--TEST LINE--", (ip, port))
                recv, svr = s.recvfrom(255)
        except socket.error, se:
            debug('test_port_status failed socket error:'+str(se[0]))
            #handle specific errors here, for now just for debug...
            ecode=se[0]
            if ecode == socket.errno.ECONNREFUSED:
                debug("test_port_status: Connection Refused")
            if ecode == socket.errno.ENETUNREACH:
                debug("test_port_status: Network unreachable")
            if ecode == socket.errno.ETIMEDOUT or ecode == "timed out":
                debug("test_port_status: Connect to "+str(ip)+":" +str(port)+ " timed out")
            raise se
        except socket.timeout, st:
            debug('test_port_status failed socket timeout')
            raise st
        finally:
            s.settimeout(None)
            s.close()
        debug('test_port_status, success')
    
    def grep(self, string,list):
        """ Remove the strings from the list that do not match the regex string"""
        expr = re.compile(string)
        return filter(expr.search,list)

    def diff(self, list1, list2):
        """Return the diff of the two lists"""
        return list(set(list1)-set(list2))
    
    def fail(self, message):
        self.critical(message)
        #self.fail_log.append(message)
        self.fail_count += 1
        if self.exit_on_fail == 1:
            raise Exception("Test step failed: "+str(message))
        else:
            return 0 
    
    def clear_fail_log(self):
        self.fail_log = []
        return
    
    def get_exectuion_time(self):
        """Returns the total execution time since the instantiation of the Eutester object"""
        return time.time() - self.start_time
       
    def clear_fail_count(self):
        """ The counter for keeping track of all the errors """
        self.fail_count = 0
        
    def sleep(self, seconds=1):
        """Convinience function for time.sleep()"""
        self.debug("Sleeping for " + str(seconds) + " seconds")
        time.sleep(seconds)

    @staticmethod
    def render_file_template(src, dest, **kwargs):
        import jinja2
        with open(src) as sfile:
            templ = jinja2.Template(sfile.read())
            with open(dest, 'w') as dfile:
                dfile.write(templ.render(kwargs))
    
    def id_generator(self, size=6, chars=string.ascii_uppercase + string.ascii_lowercase  + string.digits ):
        """Returns a string of size with random charachters from the chars array.
             size    Size of string to return
             chars   Array of characters to use in generation of the string
        """
        return ''.join(random.choice(chars) for x in range(size))
    
    @classmethod
    def printinfo(cls, func):
        '''
        Decorator to print method positional and keyword args when decorated method is called
        usage:
        @printinfo
        def myfunction(self, arg1, arg2, kwarg1=defaultval):
            stuff = dostuff(arg1, arg2, kwarg1)
            return stuff
        When the method is run it will produce debug output showing info as to how the method was called, example:
        
        myfunction(arg1=123, arg2='abc', kwarg='words)
        
        2013-02-07 14:46:58,928] [DEBUG]:(mydir/myfile.py:1234) - Starting method: myfunction()
        2013-02-07 14:46:58,928] [DEBUG]:---> myfunction(self, arg1=123, arg2=abc, kwarg='words')
        '''

        @wraps(func)
        def methdecor(*func_args, **func_kwargs):
            try:
                defaults = func.func_defaults
                kw_count = len(defaults or [])
                selfobj = None
                arg_count = func.func_code.co_argcount - kw_count
                var_names = func.func_code.co_varnames[:func.func_code.co_argcount]
                arg_names = var_names[:arg_count]
                kw_names =  var_names[arg_count:func.func_code.co_argcount]
                kw_defaults = {}
                for kw_name in kw_names: 
                    kw_defaults[kw_name] = defaults[kw_names.index(kw_name)]
                arg_string=''
                #iterate on func_args instead of arg_names to make sure we pull out self object if present
                for count, arg in enumerate(func_args):
                    if count == 0 and var_names[0] == 'self': #and if hasattr(arg, func.func_name):
                        #self was passed don't print obj addr, and save obj for later
                        arg_string += 'self'
                        selfobj = arg
                    elif count >= arg_count:
                        #Handle case where kw args are passed w/o key word as a positional arg add 
                        #Add it to the kw_defaults so it gets printed later
                        kw_defaults[var_names[count]] = arg
                    else:
                        #This is a positional arg so grab name from arg_names list
                        arg_string += ', '
                        arg_string += str(arg_names[count])+'='+str(arg)
                kw_string = ""
                for kw in kw_names:
                    kw_string += ', '+str(kw)+'='
                    if kw in func_kwargs:
                        kw_string += str(func_kwargs[kw])
                    else:
                        kw_string += str(kw_defaults[kw])
                debugstring = '\n--->('+str(os.path.basename(func.func_code.co_filename))+":"+str(func.func_code.co_firstlineno)+")Starting method: "+str(func.func_name)+'('+arg_string+kw_string+')'
                debugmethod = None
                if selfobj and hasattr(selfobj,'debug'):
                    debug = getattr(selfobj, 'debug')
                    if isinstance(debug, types.MethodType):
                        debugmethod = debug
                if debugmethod:    
                    debugmethod(debugstring)
                else:
                    print debugstring
            except Exception, e:
                print Eutester.get_traceback()
                print 'printinfo method decorator error:'+str(e)
            return func(*func_args, **func_kwargs)
        return methdecor

    def wait_for_result(self, callback, result, timeout=60, poll_wait=10, oper=operator.eq,  **callback_kwargs):
        """
        Wait for the instance to enter the state

        :param instance: Boto instance object to check the state on
        :param result: result from the call back provided that we are looking for
        :param poll_count: Number of 10 second poll intervals to wait before failure (for legacy test script support)
        :param timeout: Time in seconds to wait before failure
        :param oper: operator obj used to evaluate 'result' against callback's result. ie operator.eq, operator.ne, etc..
        :return: result upon success
        :raise: Exception when instance does not enter proper state
        """
        self.debug( "Beginning poll loop for result " + str(callback.func_name) + " to go to " + str(result) )
        start = time.time()
        elapsed = 0
        current_state =  callback(**callback_kwargs)
        ### If the instance changes state or goes to the desired state before my poll count is complete
        while( elapsed <  timeout and not oper(current_state,result) ):
            self.debug(  str(callback.func_name) + ' returned: "' + str(current_state) + '" after '
                       + str(elapsed/60) + " minutes " + str(elapsed%60) + " seconds.")
            self.sleep(poll_wait)
            current_state = callback(**callback_kwargs)
            elapsed = int(time.time()- start)
        self.debug(  str(callback.func_name) + ' returned: "' + str(current_state) + '" after '
                    + str(elapsed/60) + " minutes " + str(elapsed%60) + " seconds.")
        if not oper(current_state,result):
            raise Exception( str(callback.func_name) + " did not return " + str(operator.ne.__name__) +
                             "(" + str(result) + ") true after elapsed:"+str(elapsed))
        return current_state


    @classmethod
    def get_traceback(cls):
        '''
        Returns a string buffer with traceback, to be used for debug/info purposes. 
        '''
        try:
            out = StringIO.StringIO()
            traceback.print_exception(*sys.exc_info(),file=out)
            out.seek(0)
            buf = out.read()
        except Exception, e:
                buf = "Could not get traceback"+str(e)
        return str(buf) 
    
    def __str__(self):
        return 'got self'

    





# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on Mar 21, 2013
@author: clarkmatthew
Place holder for availability zone test specific convenience methods+objects to extend boto's zone class

'''
import eutester
from boto.ec2.zone import Zone
import re

class Vm_Type():
    def __init__(self, name,free,max, cpu, ram, disk):
        self.name = name
        self.free = free
        self.max = max
        self.cpu = cpu
        self.ram = ram
        self.disk = disk

class EuZone(Zone):
    tester = None
    vm_types = []

    @classmethod
    def make_euzone_from_zone(cls, zone, tester):
        newzone = EuZone(zone.connection)
        newzone.__dict__ = zone.__dict__
        newzone.tester = tester
        newzone.vm_types = newzone.get_all_vm_type_info()
        return newzone

    def debug(self,msg):
        self.tester.debug(msg)

    def update(self):
        super(EuZone, self).update()
        self.vm_types = self.get_all_vm_type_info()

    @eutester.Eutester.printinfo
    def get_all_vm_type_info(self):
        vm_types = []
        get_zone = [str(self.name)]
        get_zone.append('verbose')
        found = False
        try:
            myzone = self.tester.ec2.get_all_zones(zones=get_zone)
        except Exception, e:
            tb = self.tester.get_traceback()
            raise Exception(str(tb) + '\n Could not get zone:' + str(self.name) + "\n" + str(e))
        for zone in myzone:
            if zone.name == self.name:
                found = True
            if found:
                name_split = zone.name.split()
                if '|-' in name_split and len(name_split) == 2:
                    type_name = str(name_split[1])
                    state_split = zone.state.split()
                    if '/' in state_split:
                        state_split.remove('/')
                    free = int(state_split[0])
                    max  = int(state_split[1])
                    cpu = int(state_split[2])
                    ram = int(state_split[3])
                    disk = int(state_split[4])
                    it = Vm_Type(type_name,free,max,cpu,ram,disk)
                    vm_types.append(it)
                    #Remove the setattr part after dev/debug?
                    self.__setattr__('vmtype_' + str(type_name.replace('.','_')), it)
                else:
                    if not re.search('vm types', zone.name) and  zone.name != self.name:
                        break
        return vm_types

    @eutester.Eutester.printinfo
    def get_vm_types(self, name=None, free=None, max=None, cpu=None, ram=None, disk=None, refresh_types=False):
        ret_list = []
        if refresh_types or not self.vm_types:
            types_list = self.update_all_vm_type_info()
        else:
            types_list = self.vm_types

        for type in types_list:
            if name and name != type.name:
                continue
            if cpu and cpu > type.cpu:
                continue
            if ram and ram > type.ram:
                continue
            if disk and disk > type.disk:
                continue
            if free and  free > type.free:
                continue
            if max and max > type.max:
                continue
            ret_list.append(type)
        return ret_list





# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Vic Iglesias vic.iglesias@eucalyptus.com
#         

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import eutester

setup(name = "eutester",
      version = eutester.__version__,
      description = "Test Framework for AWS compatible clouds",
      long_description="Test Framework for AWS compatible clouds",
      author = "Victor Iglesias",
      author_email = "vic.iglesias@eucalyptus.com",
      url = "http://open.eucalyptus.com",
      install_requires = ['paramiko >= 1.7','boto >= 2.5.2', 'jinja2 >= 2.7', 'argparse', 'futures', 'python-dateutil',
                          'mock', 'dnspython', 'pywinrm'],
      packages = ["eutester","eucaops", "eucaweb", "testcases", "testcases.cloud_admin", "testcases.cloud_user",
                  "testcases.cloud_user.instances", "testcases.cloud_user.s3",  "testcases.cloud_user.load",  
                  "testcases.cloud_user.images", "testcases.cloud_user.images.eustore", "testcases.cloud_user.images.windows",
                  "testcases.cloud_user.ebs"],
      license = 'BSD (Simplified)',
      platforms = 'Posix; MacOS X; Windows',
      classifiers = [ 'Development Status :: 3 - Alpha',
                      'Intended Audience :: System Administrators',
                      'License :: OSI Approved :: BSD License',
                      'Operating System :: OS Independent',
                      'Topic :: System :: Systems Administration',
                      ],
      )



#!/usr/bin/python

from eucaops import Eucaops
from eutester import eupopulator
import argparse
import os

parser = argparse.ArgumentParser(prog="populate.py",
                                     description="Populate a user with various resources including snapshots,volumes,buckets,keypairs,security groups",
                                     usage="%(prog)s --credpath=<path to creds>")
parser.add_argument('--credpath',
                        help="path to user credentials"
			, required=True)
args = parser.parse_args()

tester = Eucaops(credpath=args.credpath)
pop = eupopulator.EuPopulator(tester)

pop.populate()
pop.serialize_resources(args.credpath.strip("/")  + "-before.dat")




import unittest
import inspect
import time
import gc
import argparse
import re
import sys
import os
import types
import traceback
import random
import string
from eutester.eulogger import Eulogger
from eutester.euconfig import EuConfig
import StringIO
import copy

'''
This is the base class for any test case to be included in the Eutester repo. It should include any
functionality that we expected to be repeated in most of the test cases that will be written.

Currently included:
 - Debug method
 - Allow parameterized test cases
 - Method to run test case
 - Run a list of test cases
 - Start, end and current status messages
 - Enum class for possible test results
 
Necessary to work on:
 - Argument parsing
 - Metric tracking (need to define what metrics we want
 - Standardized result summary
 - Logging standardization
 - Use docstring as description for test case
 - Standardized setUp and tearDown that provides useful/necessary cloud resources (ie group, keypair, image)
'''


class EutesterTestResult():
    '''
    standardized test results
    '''
    not_run="not_run"
    passed="passed"
    failed="failed"
    
class TestColor():
    reset = '\033[0m'
    #formats
    formats={'reset':'0',
             'bold':'1',
             'dim':'2',
             'uline':'4',
             'blink':'5',
             'reverse':'7',
             'hidden':'8',
             }
    
    foregrounds = {'black':30,
                   'red':31,
                   'green':32,
                   'yellow':33,
                   'blue':34,
                   'magenta':35,
                   'cyan':36,
                   'white':37,
                   'setasdefault':39}
    
    backgrounds = {'black':40,
                   'red':41,
                   'green':42,
                   'yellow':43,
                   'blue':44,
                   'magenta':45,
                   'cyan':46,
                   'white':47,
                   'setasdefault':49}
    
    #list of canned color schemes, for now add em as you need 'em?
    canned_colors ={'reset' : '\033[0m', #self.TestColor.get_color(fg=0)
                    'whiteonblue' : '\33[1;37;44m', #get_color(fmt=bold, fg=37,bg=44)
                    'whiteongreen' : '\33[1;37;42m',
                    'red' : '\33[31m', #TestColor.get_color(fg=31)
                    'failred' : '\033[101m', #TestColor.get_color(fg=101) 
                    'blueongrey' : '\33[1;34;47m', #TestColor.get_color(fmt=bold, fg=34, bg=47)#'\33[1;34;47m'
                    'redongrey' : '\33[1;31;47m', #TestColor.get_color(fmt=bold, fg=31, bg=47)#'\33[1;31;47m'
                    'blinkwhiteonred' : '\33[1;5;37;41m', #TestColor.get_color(fmt=[bold,blink],fg=37,bg=41)#
                    }
    
    @classmethod
    def get_color(cls,fmt=0,fg='', bg=''):
        '''
        Description: Method to return ascii color codes to format terminal output. 
        Examples:
                blinking_red_on_black = get_color('blink', 'red', 'blue')
                bold_white_fg = get_color('bold', 'white, '')
                green_fg = get_color('','green','')
                
                print bold_white_fg+"This text is bold white"+TestColor.reset
        :type fmt: color attribute
        :param fmt: An integer or string that represents an ascii color attribute. see TestColor.formats
        
        :type fg: ascii foreground attribute
        :param fg: An integer or string that represents an ascii foreground color attribute. see TestColor.foregrounds
        
        :type bg: ascii background attribute
        :param bg: An integer or string that represents an ascii background color attribute. see TestColor.backgrounds
        '''
        
        fmts=''
        if not isinstance(fmt, types.ListType):
            fmt = [fmt]
        for f in fmt:
            if isinstance(f,types.StringType):
                f = TestColor.get_format_from_string(f)
            if f:
                fmts += str(f)+';'
        if bg:
            if isinstance(bg,types.StringType):
                bg = TestColor.get_bg_from_string(bg)
            if bg:
                bg = str(bg)
        if fg:
            if isinstance(fg,types.StringType):
                fg = TestColor.get_fg_from_string(fg)
            if fg:
                fg = str(fg)+';'
        
        return '\033['+str(fmts)+str(fg)+str(bg)+'m'
    
    @classmethod
    def get_format_from_string(cls,format):
        if format in TestColor.formats:
            return TestColor.formats[format]
        else:
            return ''
        
    @classmethod
    def get_fg_from_string(cls,fg):
        if fg in TestColor.foregrounds:
            return TestColor.foregrounds[fg]
        else:
            return ''
    @classmethod
    def get_bg_from_string(cls,bg):
        if bg in TestColor.backgrounds:
            return TestColor.backgrounds[bg]
        else:
            return ''
        
    @classmethod
    def get_canned_color(cls,color):
        try:
            return TestColor.canned_colors[color]
        except:
            return ""    
    
    
    
    
class EutesterTestUnit():
    '''
    Description: Convenience class to run wrap individual methods, and run and store and access results.
    
    type method: method
    param method: The underlying method for this object to wrap, run and provide information on 
    
    type args: list of arguments
    param args: the arguments to be fed to the given 'method'
    
    type eof: boolean
    param eof: boolean to indicate whether a failure while running the given 'method' should end the test case exectution. 
    '''
    def __init__(self,method, *args, **kwargs):
        self.method = method
        self.method_possible_args = EutesterTestCase.get_meth_arg_names(self.method)
        self.args = args
        self.kwargs = kwargs 
        self.name = str(method.__name__)
        self.result=EutesterTestResult.not_run
        self.time_to_run=0
        if self.kwargs.get('html_anchors', False):
            self.anchor_id = str(str(time.ctime())
                                + self.name
                                + "_"
                                + str( ''.join(random.choice(string.ascii_uppercase + string.ascii_lowercase + string.digits) for x in range(3)))
                                + "_"
                                ).replace(" ","_")
            self.error_anchor_id = "ERROR_" + self.anchor_id
        self.description=self.get_test_method_description()
        self.eof=False
        self.error = ""
        print "Creating testunit:" + str(self.name)+", args:"
        for count, thing in enumerate(args):
            print '{0}. {1}'.format(count, thing)
        for name, value in kwargs.items():
            print '{0} = {1}'.format(name, value)
    
    @classmethod
    def create_testcase_from_method(cls, method, eof=False, *args, **kwargs):
        '''
        Description: Creates a EutesterTestUnit object from a method and set of arguments to be fed to that method
        
        type method: method
        param method: The underlying method for this object to wrap, run and provide information on
        
        type args: list of arguments
        param args: the arguments to be fed to the given 'method'
        '''
        testunit =  EutesterTestUnit(method, args, kwargs)
        testunit.eof = eof
        return testunit
       
    def set_kwarg(self,kwarg,val):
        self.kwargs[kwarg]=val
         
    def get_test_method_description(self):
        '''
        Description:
        Attempts to derive test unit description for the registered test method.
        Keys off the string "Description:" preceded by any amount of white space and ending with either
        a blank line or the string "EndDescription". This is used in debug output when providing info to the
        user as to the method being run as a testunit's intention/description.  
        '''

        desc = "\nMETHOD:"+str(self.name) + ", TEST DESCRIPTION:\n"
        ret = []
        add = False
        try:
            doc = str(self.method.__doc__)
            if not doc or not re.search('Description:',doc):
                try:
                    desc = desc+"\n".join(self.method.im_func.func_doc.title().splitlines())
                except:pass
                return desc
            has_end_marker = re.search('EndDescription', doc)
            
            for line in doc.splitlines():
                line = line.lstrip().rstrip()
                if re.search('^Description:',line.lstrip()):
                    add = True
                if not has_end_marker:
                    if not re.search('\w',line):
                        if add:
                            break
                        add = False
                else:
                    if re.search('^EndDescription'):
                        add = False
                        break
                if add:
                    ret.append(line)
        except Exception, e:
            print('get_test_method_description: error'+str(e))
        if ret:
            desc = desc+"\n".join(ret)
        return desc
    
    def run(self, eof=None):
        '''
        Description: Wrapper which attempts to run self.method and handle failures, record time.
        '''
        if eof is None:
            eof = self.eof
        for count, thing in enumerate(self.args):
            print 'ARG:{0}. {1}'.format(count, thing)
        for name, value in self.kwargs.items():
            print 'KWARG:{0} = {1}'.format(name, value)
        
        try:
            start = time.time()
            if not self.args and not self.kwargs:
                ret = self.method()
            else:
                ret = self.method(*self.args, **self.kwargs)
            self.result=EutesterTestResult.passed
            return ret
        except SkipTestException, se:
            print TestColor.get_canned_color('failred') + \
                  "TESTUNIT SKIPPED:" + str(self.name) + "\n" + str(se) + TestColor.reset
            self.error = str(se)
            self.result = EutesterTestResult.not_run
        except Exception, e:
            buf = '\nTESTUNIT FAILED: ' + self.name
            if self.kwargs.get('html_anchors',False):
                buf += "<font color=red> Error in test unit '" + self.name + "':\n"
            out = StringIO.StringIO()
            traceback.print_exception(*sys.exc_info(),file=out)
            out.seek(0)
            buf += out.read()
            if self.kwargs.get('html_anchors',False):
                buf += ' </font>'
                print '<a name="' + str(self.error_anchor_id) + '"></a>'
            print TestColor.get_canned_color('failred') + buf + TestColor.reset
            self.error = str(e)
            self.result = EutesterTestResult.failed
            if eof:
                raise e
            else:
                pass
        finally:
            self.time_to_run = int(time.time()-start)
        
                
class EutesterTestCase(unittest.TestCase):
    color = TestColor()

    def __init__(self,name=None, debugmethod=None, log_level='debug', logfile=None, logfile_level='debug'):
        return self.setuptestcase(name=name, debugmethod=debugmethod, logfile=logfile, logfile_level=logfile_level)
        
    def setuptestcase(self,
                      name=None,
                      debugmethod=None,
                      use_default_file=False,
                      default_config='eutester.conf',
                      log_level='debug',
                      logfile=None,
                      logfile_level='debug'):
        self.name = self._testMethodName = name
        self.log_level = log_level
        self.logfile = logfile
        self.logfile_level = logfile_level
        if not self.name:
            callerfilename=inspect.getouterframes(inspect.currentframe())[1][1]
            self.name = os.path.splitext(os.path.basename(callerfilename))[0]  
            self._testMethodName = self.name
            print "setuptestname:"+str(name)
        if not hasattr(self,'args'): self.args=argparse.Namespace()
        self.debugmethod = debugmethod
        if not self.debugmethod:
            self.setup_debugmethod(logfile=self.logfile, logfile_level=self.logfile_level)
        #For QA output add preformat tag
        self.debug('<pre>')
        if not hasattr(self,'testlist'): self.testlist = []
        self.list = None
        if not hasattr(self,'configfiles'): self.configfiles=[]
        self.default_config = default_config 
        self.use_default_file = use_default_file
        if use_default_file:
            #first add $USERHOME/.eutester/eutester.conf if it exists
            self.default_config=self.get_default_userhome_config(fname=default_config)
            if self.default_config:
                self.configfiles.append(self.default_config)
        self.show_self()
                
    def compile_all_args(self):
        self.setup_parser()
        self.get_args()


                                   
    def setup_parser(self,
                   testname=None, 
                   description=None,
                   emi=True,
                   zone=True,
                   vmtype=True,
                   keypair=True,
                   credpath=True,
                   password=True,
                   config=True,
                   configblocks=True,
                   ignoreblocks=True,
                   color=True,
                   testlist=True,
                   userdata=True,
                   instance_user=True,
                   stdout_log_level=True,
                   logfile_level=True,
                   logfile=True,
                   instance_password=True,
                   region=True):
        '''
        Description: Convenience method to setup argparse parser and some canned default arguments, based
        upon the boolean values provided. For each item marked as 'True' this method will add pre-defined 
        command line arguments, help strings and default values. This will then be available by the end script 
        as an alternative to recreating these items on a per script bassis. 
    
        :type testname: string
        :param testname: Name used for argparse (help menu, etc.)
        
        :type description: string
        :param description: Description used for argparse (help menu, etc.)
        
        :type emi: boolean
        :param emi: Flag to present the emi command line argument/option for providing an image emi id via the cli
        
        :type zone: boolean
        :param zone: Flag to present the zone command line argument/option for providing a zone via the cli
        
        :type vmtype: boolean
        :param vmtype: Flag to present the vmtype command line argument/option for providing a vmtype via the cli
        
        :type keypair: boolean
        :param kepair: Flag to present the keypair command line argument/option for providing a keypair via the cli
        
        :type credpath: boolean
        :param credpath: Flag to present the credpath command line argument/option for providing a local path to creds via the cli
        
        :type password: boolean
        :param password: Flag to present the password command line argument/option for providing password 
        used in establishing machine ssh sessions
        
        :type config: boolean
        :param config: Flag to present the config file command line argument/option for providing path to config file
        
        :type configblocks: string list
        :param configblocks: Flag to present the configblocks command line arg/option used to provide list of 
                             configuration blocks to read from
                             Note: By default if a config file is provided the script will only look for blocks; 'globals', and the filename of the script being run.
        
        :type ignoreblocks: string list
        :param ignoreblocks: Flag to present the configblocks command line arg/option used to provide list of 
                             configuration blocks to ignore if present in configfile
                             Note: By default if a config file is provided the script will look for blocks; 'globals', and the filename of the script being run
 
        :type testlist: string list
        :param testlist: Flag to present the testlist command line argument/option for providing a list of testnames to run
        
        :type userdata: boolean
        :param userdata: Flag to present the userdata command line argument/option for providing userdata to instance(s) within test
        
        :type instance_user: boolean
        :param instance_user: Flag to present the instance_user command line argument/option for providing an ssh username for instance login via the cli
        
        :type instance_password: boolean
        :param instance_password: Flag to present the instance_password command line argument/option for providing a ssh password for instance login via the cli
        
        :type use_color: flag
        :param use_color: Flag to enable/disable use of ascci color codes in debug output.

        :param stdout_log_level: boolean flag to present the --log_leveel command line option to set stdout log level

        :param logfile_level: boolean flag to present the --logfile_level command line option to set the log file log level

        :param logfile: boolean flag to present the --logfile command line option to set the log file path to log to

        :param region: boolean flag to present the --region command line option to set the region for the test to use
        '''
        
        testname = testname or self.name 
        
        description = description or "Test Case Default Option Parser Description"
        #create parser
        parser = argparse.ArgumentParser( prog=testname, description=description)
        #add some typical defaults:
        if emi:
            parser.add_argument('--emi', 
                                help="pre-installed emi id which to execute these tests against", default=None)
        if credpath:
            parser.add_argument('--credpath', 
                                help="path to credentials", default=None)
        if password:
            parser.add_argument('--password', 
                                help="password to use for machine root ssh access", default=None)
        if config:
            parser.add_argument('--config',
                                help='path to config file', default=None)
        if configblocks:
            parser.add_argument('--configblocks', nargs='+',
                                help="Config sections/blocks in config file to read in", default=[])
        if ignoreblocks:
            parser.add_argument('--ignoreblocks', nargs='+',
                                help="Config blocks to ignore, ie:'globals', 'my_scripts_name', etc..", default=[])
        if testlist:
            parser.add_argument('--tests', nargs='+', 
                                help="test cases to be executed", default = [])  
        if keypair:
            parser.add_argument('--keypair',
                                help="Keypair to use in this test", default=None)
        if zone:
            parser.add_argument('--zone',
                                help="Zone to use in this test", default=None)
        if vmtype:
            parser.add_argument('--vmtype',
                                help="Virtual Machine Type to use in this test", default='c1.medium')
        if userdata:
            parser.add_argument('--user-data',
                                help="User data string to provide instance run within this test", default=None)
        if instance_user:
            parser.add_argument('--instance-user',
                                help="Username used for ssh login. Default:'root'", default='root')
        if instance_password:
            parser.add_argument('--instance-passsword',
                                help="Password used for ssh login. When value is 'None' ssh keypair will be used and not username/password, default:'None'", default=None)
        if region:
            parser.add_argument('--region',
                help="Use AWS region instead of Eucalyptus", default=None)
        if color:
            parser.add_argument('--use_color', dest='use_color', action='store_true', default=False)
        if stdout_log_level:
            parser.add_argument('--log_level',
                                help="log level for stdout logging", default='debug')
        if logfile:
            parser.add_argument('--logfile',
                                help="file path to log to (in addtion to stdout", default=None)
        if logfile_level:
            parser.add_argument('--logfile_level',
                                help="log level for log file logging", default='debug')
        parser.add_argument('--html-anchors', dest='html_anchors', action='store_true',
                                help="Print HTML anchors for jumping through test results", default=False)
        self.parser = parser  
        return parser
    
    def disable_color(self):
        self.set_arg('use_color', False)
        self.use_color = False
    
    def enable_color(self):
        self.set_arg('use_color', True)
        self.use_color = True
        

    def setup_debugmethod(self, testcasename=None, log_level=None, logfile=None, logfile_level=None):
        print "setup_debugmethod: \ntestcasename:"+ str(testcasename) \
                                + '\nlog_level:'+str(log_level) \
                                + '\nlogfile:' +str(logfile) \
                                + '\nlogfile_level:' +str(logfile_level)
        name = testcasename or self.name

        if not logfile and self.has_arg('logfile'):
            logfile = self.args.logfile
        logfile = logfile or self.logfile

        if self.has_arg('logfile_level'):
            logfile_level = self.args.logfile_level
        logfile_level = logfile_level or self.logfile_level or 'debug'

        if not log_level and self.has_arg('log_level'):
            log_level = self.args.log_level
        log_level = log_level or self.log_level or 'debug'

        print "Starting setup_debugmethod, name:"+str(name)
        print "After populating... setup_debugmethod: testcasename:"+ str(testcasename) \
              + 'log_level:'+str(log_level) \
              + 'logfile:' +str(logfile) \
              + 'logfile_level:' +str(logfile_level)
        if not name:
            if hasattr(self,'name'):
                if isinstance(self.name, types.StringType):
                    name = self.name
            else:
                name = 'EutesterTestCase'
        self.logger = Eulogger(identifier=str(name),stdout_level=log_level, logfile=logfile, logfile_level='debug')
        self.debugmethod = self.logger.log.debug
        if not self.has_arg('logger'):
            self.add_arg('logger',self.logger)
        if not self.has_arg('debug_method'):
            self.add_arg('debug_method', self.debug)

    def debug(self,msg,traceback=1,color=None, linebyline=True):
        '''
        Description: Method for printing debug
        
        type msg: string
        param msg: Mandatory string buffer to be printed in debug message
        
        type traceback: integer
        param traceback: integer value for what frame to inspect to derive the originating method and method line number
        
        type color: TestColor color
        param color: Optional ascii text color scheme. See TestColor for more info. 
        '''
        try:
            if not self.debugmethod:
                self.setup_debugmethod()
        except:
            self.setup_debugmethod()

        if self.has_arg("use_color"):
            self.use_color = bool(self.args.use_color)
        else:
            self.use_color = False
            
        colorprefix=""
        colorreset=""
        #if a color was provide
        if color and self.use_color:
            colorprefix = TestColor.get_canned_color(color) or color
            colorreset = str(TestColor.get_canned_color('reset'))
        msg = str(msg)       
        curframe = None    
        curframe = inspect.currentframe(traceback)
        lineno = curframe.f_lineno
        self.curframe = curframe
        frame_code  = curframe.f_code
        frame_globals = curframe.f_globals
        functype = type(lambda: 0)
        funcs = []
        for func in gc.get_referrers(frame_code):
            if type(func) is functype:
                if getattr(func, "func_code", None) is frame_code:
                    if getattr(func, "func_globals", None) is frame_globals:
                        funcs.append(func)
                        if len(funcs) > 1:
                            return None
            cur_method= funcs[0].func_name if funcs else ""
        if linebyline:
            for line in msg.split("\n"):
                self.debugmethod("("+str(cur_method)+":"+str(lineno)+"): "+colorprefix+line.strip()+colorreset )
        else:
            self.debugmethod("("+str(cur_method)+":"+str(lineno)+"): "+colorprefix+str(msg)+colorreset )

    def run_test_list_by_name(self, list, eof=None):
        unit_list = []
        for test in list:
            unit_list.append( self.create_testunit_by_name(test) )

        ### Run the EutesterUnitTest objects
        return self.run_test_case_list(unit_list,eof=eof)

    def create_testunit_from_method(self,method, *args, **kwargs):
        '''
        Description: Convenience method calling EutesterTestUnit. 
                     Creates a EutesterTestUnit object from a method and set of arguments to be fed to that method
        
        :type method: method
        :param method: The underlying method for this object to wrap, run and provide information on
        
        :type eof: boolean
        :param eof: Boolean to indicate whether this testunit should cause a test list to end of failure
        
        :type autoarg: boolean
        :param autoarg: Boolean to indicate whether to autopopulate this testunit with values from global testcase.args
        
        :type args: list of positional arguments
        :param args: the positional arguments to be fed to the given testunit 'method'
        
        :type kwargs: list of keyword arguements 
        :param kwargs: list of keyword 
        
        :rtype: EutesterTestUnit
        :returns: EutesterTestUnit object
        '''   
        eof=False
        autoarg=True
        methvars = self.get_meth_arg_names(method)
        #Pull out value relative to this method, leave in any that are intended to be passed through
        if 'autoarg' in kwargs:
            if 'autoarg' in methvars:
                autoarg = kwargs['autoarg']
            else:
                autoarg = kwargs.pop('autoarg')
        if 'eof' in kwargs:
            if 'eof' in methvars:
                eof = kwargs['eof']
            else:
                eof = kwargs.pop('eof')
        ## Only pass the arg if we need it otherwise it will print with all methods/testunits
        if self.args.html_anchors:
            testunit = EutesterTestUnit(method, *args, html_anchors=self.args.html_anchors ,**kwargs)
        else:
            testunit = EutesterTestUnit(method, *args, **kwargs)
        testunit.eof = eof
        #if autoarg, auto populate testunit arguements from local testcase.args namespace values
        if autoarg:
            self.populate_testunit_with_args(testunit)
        return testunit 
    
    def status(self,msg,traceback=2, b=1,a=0 ,testcolor=None):
        '''
        Description: Convenience method to format debug output
        
        :type msg: string
        :param msg: The string to be formated and printed via self.debug
        
        :type traceback: integer
        :param traceback: integer value for what frame to inspect to derive the originating method and method line number
        
        :type b: integer
        :param b:number of blank lines to print before msg
        
        :type a: integer
        :param a:number of blank lines to print after msg
        
        :type testcolor: TestColor color
        :param testcolor: Optional TestColor ascii color scheme
        '''
        alines = ""
        blines = ""
        for x in xrange(0,b):
            blines=blines+"\n"
        for x in xrange(0,a):
            alines=alines+"\n"
        line = "-------------------------------------------------------------------------"
        out = blines+line+"\n"+msg+"\n"+line+alines
        self.debug(out, traceback=traceback, color=testcolor,linebyline=False)  
        
    def startmsg(self,msg=""):
        self.status(msg, traceback=3,testcolor=TestColor.get_canned_color('whiteonblue'))
        
    def endsuccess(self,msg=""):
        msg = "- UNIT ENDED - " + msg
        self.status(msg, traceback=2,a=1, testcolor=TestColor.get_canned_color('whiteongreen'))

    def errormsg(self,msg=""):
        msg = "- ERROR - " + msg
        self.status(msg, traceback=2,a=1,testcolor=TestColor.get_canned_color('failred'))

    def endfailure(self,msg="" ):
        msg = "- FAILED - " + msg
        self.status(msg, traceback=2,a=1,testcolor=TestColor.get_canned_color('failred'))
    
    def resultdefault(self,msg,printout=True,color='blueongrey'):
        if printout:
            self.debug(msg,traceback=2,color=TestColor.get_canned_color('blueongrey'),linebyline=False)
        msg = self.format_line_for_color(msg, color)
        return msg
    
    def resultfail(self,msg,printout=True, color='redongrey'):
        if printout:
            self.debug(msg,traceback=2, color=TestColor.get_canned_color('redongrey'),linebyline=False)
        msg = self.format_line_for_color(msg, color)
        return msg
        
    def resulterr(self,msg,printout=True,color='failred'):
        if printout:
            self.debug(msg,traceback=2, color=TestColor.get_canned_color(color),linebyline=False)
        msg = self.format_line_for_color(msg, color)
        return msg
    
    def format_line_for_color(self,msg,color):
        if not self.use_color:
            return msg
        end=""
        if msg.endswith('\n'):
            msg = msg.rstrip()
            end="\n"
        msg = TestColor.get_canned_color(color)+str(msg)+TestColor.reset+end
        return msg
    
    def get_pretty_args(self,testunit):
        '''
        Description: Returns a string buf containing formated arg:value for printing later
        
        :type: testunit: Eutestcase.eutestertestunit object
        :param: testunit: A testunit object for which the namespace args will be used
        
        :rtype: string
        :returns: formated string containing args and their values.  
        '''
        
        buf =  "\nEnd on Failure:" +str(testunit.eof)
        buf += "\nPassing ARGS:"
        if not testunit.args and not testunit.kwargs:
            buf += '\"\"\n'
        else:
            buf += "\n---------------------\n"
            varnames = self.get_meth_arg_names(testunit.method)
            if testunit.args:
                for count,arg in enumerate(testunit.args):
                    buf += str(varnames[count+1])+" : "+str(arg)+"\n"
            if testunit.kwargs:
                for key in testunit.kwargs:
                    buf += str(key)+" : "+str(testunit.kwargs[key])+"\n"
            buf += "---------------------\n"
        return buf
    
    def run_test_case_list(self, list, eof=False, clean_on_exit=True, printresults=True):
        '''
        Desscription: wrapper to execute a list of ebsTestCase objects
        
        :type list: list
        :param list: list of EutesterTestUnit objects to be run
        
        :type eof: boolean
        :param eof: Flag to indicate whether run_test_case_list should exit on any failures. If this is set to False it will exit only when
                    a given EutesterTestUnit fails and has it's eof flag set to True. 
        
        :type clean_on_exit: boolean
        :param clean_on_exit: Flag to indicate if clean_on_exit should be ran at end of test list execution. 
        
        :type printresults: boolean
        :param printresults: Flag to indicate whether or not to print a summary of results upon run_test_case_list completion. 
        
        :rtype: integer
        :returns: integer exit code to represent pass/fail of the list executed. 
        '''
        self.testlist = list 
        start = time.time()
        tests_ran=0
        test_count = len(list)
        try:
            for test in list:
                tests_ran += 1
                self.print_test_unit_startmsg(test)
                try:
                    test.run(eof=eof or test.eof)
                except Exception, e:
                    self.debug('Testcase:'+ str(test.name)+' error:'+str(e))
                    if eof or (not eof and test.eof):
                        self.endfailure(str(test.name))
                        raise e
                    else:
                        self.endfailure(str(test.name))
                else:
                    self.endsuccess(str(test.name))
                self.debug(self.print_test_list_short_stats(list))
                        
        finally:
            elapsed = int(time.time()-start)
            msgout =  "RUN TEST CASE LIST DONE:\n"
            msgout += "Ran "+str(tests_ran)+"/"+str(test_count)+" tests in "+str(elapsed)+" seconds\n"
            
            if printresults:
                try:
                    self.debug("Printing pre-cleanup results:")
                    msgout += self.print_test_list_results(list=list,printout=False)
                    self.status(msgout)
                except:pass
            try:
                 if clean_on_exit:
                    cleanunit = self.create_testunit_from_method(self.clean_method)
                    list.append(cleanunit)
                    try:
                        self.print_test_unit_startmsg(cleanunit)
                        cleanunit.run()
                    except Exception, e:
                        out = StringIO.StringIO()
                        traceback.print_exception(*sys.exc_info(),file=out)
                        out.seek(0)
                        self.debug("Failure in cleanup: " + str(e) + "\n" + out.read())
                    if printresults:
                        msgout = self.print_test_list_results(list=list,printout=False)
                        self.status(msgout)
            except: 
                pass
            self.testlist = copy.copy(list)
            passed = 0
            failed = 0
            not_run = 0
            for test in list:
                if test.result == EutesterTestResult.passed:
                    passed += 1
                if test.result == EutesterTestResult.failed:
                    failed += 1
                if test.result == EutesterTestResult.not_run:
                    not_run += 1
            total = passed + failed + not_run
            print "passed:"+str(passed)+" failed:" + str(failed) + " not_run:" + str(not_run) + " total:"+str(total)
            if failed:
                return(1)
            else:
                return(0)

    def print_test_unit_startmsg(self,test):
        startbuf = ''
        if self.args.html_anchors:
            link = '<a name="' + str(test.anchor_id) + '"></a>\n'
            startbuf += '<div id="myDiv" name="myDiv" title="Example Div Element" style="color: #0900C4; font: Helvetica 12pt;border: 1px solid black;">'
            startbuf += str(link)
        startbuf += "STARTING TESTUNIT: " + test.name
        argbuf = self.get_pretty_args(test)
        startbuf += str(test.description)+str(argbuf)
        startbuf += 'Running list method: "'+str(self.print_testunit_method_arg_values(test))+'"'
        if self.args.html_anchors:
            startbuf += '\n </div>'
        self.startmsg(startbuf)
    
    def has_arg(self,arg):
        '''
        Description: If arg is present in local testcase args namespace, will return True, else False
        
        :type arg: string
        :param arg: string name of arg to check for.

        :rtype: boolean
        :returns: True if arg is present, false if not
        '''
        arg = str(arg)
        if hasattr(self,'args'):
            if self.args and (arg in self.args):
                return True
        return False
         
    def get_arg(self,arg):
        '''
        Description: Fetchs the value of an arg within the local testcase args namespace. If the arg
        does not exist, None will be returned. 
        
        :type arg: string
        :param arg: string name of arg to get.
        
        :rtype: value
        :returns: Value of arguement given, or None if not found
        '''
        if self.has_arg(arg):
            return getattr(self.args,str(arg))
        return None
    
    def add_arg(self,arg,value):
        '''
        Description: Adds an arg 'arg'  within the local testcase args namespace and assigns it 'value'. 
        If arg exists already in testcase.args, then an exception will be raised. 
        
        :type arg: string
        :param arg: string name of arg to set. 
        
        :type value: value
        :param value: value to set arg to
        '''
        if self.has_arg(arg):
            raise Exception("Arg"+str(arg)+'already exists in args')
        else:
            self.args.__setattr__(arg,value)
    
    def set_arg(self,arg, value):
        '''
        Description: Sets an arg 'arg'  within the local testcase args namespace to 'value'. 
        If arg does not exist in testcase.args, then it will be created. 
        
        :type arg: string
        :param arg: string name of arg to set. 
        
        :type value: value
        :param value: value to set arg to
        '''
        
        if self.has_arg(arg):
            new = argparse.Namespace()
            for val in self.args._get_kwargs():
                if arg != val[0]:
                    new.__setattr__(val[0],val[1])
            new.__setattr__(arg,value)
            self.args = new
        else:
            self.args.__setattr__(arg,value)
    
    
    def clean_method(self):
        raise Exception("Clean_method was not implemented. Was run_list using clean_on_exit?")

    def print_test_list_results(self,list=None, printout=True, printmethod=None):
        '''
        Description: Prints a formated list of results for a list of EutesterTestUnits
        
        :type list: list
        :param list: list of EutesterTestUnits
        
        :type printout: boolean
        :param printout: boolean to flag whether to print using printmethod or self.debug, 
                         or to return a string buffer representing the results output 
        
        :type printmethod: method
        :param printmethod: method to use for printing test result output. Default is self.debug
        '''

        buf = "\nTESTUNIT LIST SUMMARY FOR " + str(self.name) + "\n"
        if list is None:
            list=self.testlist
        if not list:
            raise Exception("print_test_list_results, error: No Test list provided")
        if printmethod is None:
            printmethod = lambda msg: self.debug(msg,linebyline=False)
        printmethod("Test list results for testcase:"+str(self.name))

        for testunit in list:
            buf += self.resultdefault("\n"+ self.getline(80)+"\n", printout=False)
            #Ascii mark up errors using pmethod() so errors are in bold/red, etc...
            pmethod = self.resultfail if not testunit.result == EutesterTestResult.passed else self.resultdefault
            test_summary_line = str(" ").ljust(20) + str("| RESULT: " + str(testunit.result)).ljust(20) + "\n" +\
                                str(" ").ljust(20) + "| TEST NAME: " + str(testunit.name) + "\n" + \
                                str(" ").ljust(20) + str("| TIME : " + str(testunit.time_to_run))


            buf += pmethod(str(test_summary_line),printout=False)
            buf += pmethod("\n" + str(" ").ljust(20) + "| ARGS: "
                           + str(self.print_testunit_method_arg_values(testunit)), printout=False)
            #Print additional line showing error in the failed case...
            if testunit.result == EutesterTestResult.failed:
                    err_sum = "\n".join(str(testunit.error).splitlines()[0:3])
                    test_error_line = 'ERROR:('+str(testunit.name)+'): '\
                                      + str(err_sum) \
                                      + '\n'
                    buf += "\n"+str(self.resulterr(test_error_line, printout=False))
            if testunit.result == EutesterTestResult.not_run:
                err_sum = "\n".join(str(testunit.error).splitlines()[0:3])
                test_error_line = 'NOT_RUN:('+str(testunit.name)+'): ' \
                                  + str(err_sum) \
                                  + '\n'
                buf += "\n"+str(self.resulterr(test_error_line, printout=False))
        buf += self.resultdefault("\n"+ self.getline(80)+"\n", printout=False)
        buf += str(self.print_test_list_short_stats(list))
        buf += "\n"
        if printout:
            printmethod(buf)
        else:
            return buf

        
    def print_test_list_short_stats(self,list,printmethod=None):
        results={}
        mainbuf = "RESULTS SUMMARY FOR '"+str(self.name)+"':\n"
        fieldsbuf = ""
        resultsbuf= ""
        total = 0 
        elapsed = 0
        #initialize a dict containing all the possible defined test results
        fields = dir(EutesterTestResult)
        for fieldname in fields[2:len(fields)]:
            results[fieldname]=0
        #increment values in results dict based upon result of each testunit in list
        for testunit in list:
            total += 1
            elapsed += testunit.time_to_run
            results[testunit.result] += 1
        fieldsbuf += str('| TOTAL').ljust(10)
        resultsbuf += str('| ' + str(total)).ljust(10)
        for field in results:
            fieldsbuf += str('| ' + field.upper()).ljust(10)
            resultsbuf += str('| ' + str(results[field])).ljust(10)
        fieldsbuf += str('| TIME_ELAPSED').ljust(10)
        resultsbuf += str('| '+str(elapsed)).ljust(10)
        mainbuf += "\n"+self.getline(len(fieldsbuf))+"\n"
        mainbuf += fieldsbuf
        mainbuf += "\n"+self.getline(len(fieldsbuf))+"\n"
        mainbuf += resultsbuf
        mainbuf += "\n"+self.getline(len(fieldsbuf))+"\n"
        if printmethod:
            printmethod(mainbuf)
        return mainbuf
    
    @classmethod  
    def get_testunit_method_arg_dict(cls,testunit):
        argdict={}
        spec = inspect.getargspec(testunit.method)
        if isinstance(testunit.method,types.FunctionType):
            argnames = spec.args
        else:
            argnames = spec.args[1:len(spec.args)]
        defaults = spec.defaults or []
        #Initialize the return dict
        for argname in argnames:
            argdict[argname]='<!None!>'
        #Set the default values of the testunits method
        for x in xrange(0,len(defaults)):
            argdict[argnames.pop()]=defaults[len(defaults)-x-1]
        #Then overwrite those with the testunits kwargs values
        for kwarg in testunit.kwargs:
            argdict[kwarg]=testunit.kwargs[kwarg]
        #then add the positional args in if they apply...
        for count, value in enumerate(testunit.args):
            argdict[argnames[count]]=value
        return argdict
    
    @classmethod
    def print_testunit_method_arg_values(cls,testunit):
        buf = testunit.name+"("
        argdict = EutesterTestCase.get_testunit_method_arg_dict(testunit)
        for arg in argdict:
            buf += str(arg)+":"+str(argdict[arg])+","
        buf = buf.rstrip(',')
        buf += ")"
        return buf
        
        
    def getline(self,len):
        buf = ''
        for x in xrange(0,len):
            buf += '-'
        return buf
    
    def run_method_by_name(self,name, obj=None, *args, **kwargs):
        '''
        Description: Find a method within an instance of obj and run that method with either args/kwargs provided or
        any self.args which match the methods varname. 
        
        :type name: string
        :param name: Name of method to look for within instance of object 'obj'
        
        :type obj: class instance
        :param obj: Instance type, defaults to self testcase object
        
        :type args: positional arguements
        :param args: None or more positional arguments to be passed to method to be run
        
        :type kwargs: keyword arguments
        :param kwargs: None or more keyword arguements to be passed to method to be run
        '''
        obj = obj or self
        meth = getattr(obj,name)
        return self.do_with_args(meth, *args, **kwargs)
        
        
        
    
    def create_testunit_by_name(self, name, obj=None, eof=True, autoarg=True, *args,**kwargs ):
        '''
        Description: Attempts to match a method name contained with object 'obj', and create a EutesterTestUnit object from that method and the provided
        positional as well as keyword arguments provided. 
        
        :type name: string
        :param name: Name of method to look for within instance of object 'obj'
        
        :type obj: class instance
        :param obj: Instance type, defaults to self testcase object
        
        :type args: positional arguements
        :param args: None or more positional arguments to be passed to method to be run
        
        :type kwargs: keyword arguments
        :param kwargs: None or more keyword arguements to be passed to method to be run
        '''
        eof=False
        autoarg=True
        obj = obj or self
        meth = getattr(obj,name)
        methvars = self.get_meth_arg_names(meth)


        
        #Pull out value relative to this method, leave in any that are intended to be passed through
        if 'autoarg' in kwargs:
            if 'autoarg' in methvars:
                autoarg = kwargs['autoarg']
            else:
                autoarg = kwargs.pop('autoarg')
        if 'eof' in kwargs:
            if 'eof' in methvars:
                eof = kwargs['eof']
            else:
                eof = kwargs.pop('eof')
        if 'obj' in kwargs:
            if 'obj' in methvars:
                obj = kwargs['obj']
            else:
                obj = kwargs.pop('obj')

        testunit = EutesterTestUnit(meth, *args, **kwargs)
        testunit.eof = eof

        #if autoarg, auto populate testunit arguements from local testcase.args namespace values
        if autoarg:
            self.populate_testunit_with_args(testunit)
            
        return testunit

        
    def get_args(self,use_cli=True, file_sections=[]):
        '''
        Description: Method will attempt to retrieve all command line arguments presented through local 
        testcase's 'argparse' methods, as well as retrieve all EuConfig file arguments. All arguments 
        will be combined into a single namespace object held locally at 'testcase.args'. Note: cli arg 'config'
        must be provided for config file valus to be store in self.args. 
        
        :type use_cli: boolean
        :param use_cli: Boolean to indicate whether or not to create and read from a cli argparsing object
        
        :type use_default_file: boolean
        :param use_default_files: Boolean to indicate whether or not to read default config file at $HOME/.eutester/eutester.conf (not indicated by cli)
        
        :type sections: list
        :param sections: list of EuConfig sections to read configuration values from, and store in self.args.
        
        :rtype: arparse.namespace obj
        :returns: namespace object with values from cli and config file arguements 
        '''
        configfiles=[]
        args=None
        #build out a namespace object from the config file first
        cf = argparse.Namespace()
        
        
        if (hasattr(self, 'use_default_file') and self.use_default_file) and \
                (hasattr(self, 'use_default_config') and self.default_config):
            try:
                configfiles.append(self.default_config)
            except Exception, e:
                self.debug("Unable to read config from file: " + str(e))

        #Setup/define the config file block/sections we intend to read from
        confblocks = file_sections or ['MEMO','globals']
        if self.name:
            confblocks.append(self.name)
        
        if use_cli:
            #See if we have CLI args to read
            if not hasattr(self,'parser') or not self.parser:
                self.setup_parser()
            #first get command line args to see if there's a config file
            cliargs = self.parser.parse_args()
            
        #if a config file was passed, combine the config file and command line args into a single namespace object
        if cliargs:
            #Check to see if there's explicit config sections to read
            if 'configblocks' in cliargs.__dict__:
                confblocks = confblocks +cliargs.configblocks
            #Check to see if there's explicit config sections to ignore
            if 'ignoreblocks' in cliargs.__dict__:
                for block in cliargs.ignoreblocks:
                    if block in confblocks:
                        confblocks.remove(block)
            
            #if a file or list of config files is specified add it to our list...
            if ('config' in cliargs.__dict__) and  cliargs.config:
                for cfile in str(cliargs.config).split(','):
                    if not cfile in configfiles:
                        configfiles.append(cfile)
            #legacy support for config, configfile config_file arg names...
            if ('config_file' in cliargs.__dict__) and  cliargs.config:
                for cfile in str(cliargs.config).split(','):
                    if not cfile in configfiles:
                        configfiles.append(cfile)
            #legacy support for config, configfile config_file arg names...
            if ('configfile' in cliargs.__dict__) and  cliargs.config:
                for cfile in str(cliargs.config).split(','):
                    if not cfile in configfiles:
                        configfiles.append(cfile)

        #store config block list for debug purposes
        cf.__setattr__('configsections',copy.copy(confblocks))
        
        #create euconfig configparser objects from each file. 
        euconfigs = []
        self.configfiles = configfiles
        try:
            for configfile in configfiles:
                euconfigs.append(EuConfig(filename=configfile))
        except Exception, e:
            self.debug("Unable to read config from file: " + str(e))

        for conf in euconfigs:
            cblocks = copy.copy(confblocks)
            #if MEMO field in our config block add it first if to set least precedence
            if 'MEMO' in cblocks:
                if conf.config.has_section('MEMO'):
                    for item in conf.config.items('MEMO'):
                        cf.__setattr__(str(item[0]), item[1])
                    cblocks.remove('MEMO')
            
            #If globals are still in our confblocks, add globals first if the section is present in config
            if 'globals' in cblocks:
                if conf.config.has_section('globals'):
                    for item in conf.config.items('globals'):
                        cf.__setattr__(str(item[0]), item[1])
                    cblocks.remove('globals')
            
            #Now iterate through remaining config block in file and add to args...
            for section in confblocks:
                if conf.config.has_section(section):
                    for item in conf.config.items(section):
                        cf.__setattr__(str(item[0]), item[1])
                        
        if cliargs:
            #Now make sure any conflicting args provided on the command line take precedence over config file args
            for val in cliargs._get_kwargs():
                if (not val[0] in cf ) or (val[1] is not None):
                    cf.__setattr__(str(val[0]), val[1])
            args = cf
        #Legacy script support: level set var names for config_file vs configfile vs config and credpath vs cred_path
        try:
            if 'config' in args:
                args.config_file = args.config
                args.configfile = args.config

        except: pass
        try:
            args.cred_path = args.credpath
        except: pass
        self.args = args
        #finally add the namespace args to args for populating other testcase objs from this one
        if not self.has_arg('args'): args.__setattr__('args',copy.copy(args))
        #Refresh our debug method(s) in the case the args provided give instruction on logging
        self.setup_debugmethod()
        self.show_self()
        return args
    
        
    
    def get_default_userhome_config(self,fname='eutester.conf'):
        '''
        Description: Attempts to fetch the file 'fname' from the current user's home dir. 
        Returns path to the user's home dir default eutester config file. 
        
        :type fname: string
        :param fname: the eutester default config file name
        
        :rtype: string
        :returns: string representing the path to 'fname', the default eutester conf file. 
        
        '''
        try:
            def_path = os.getenv('HOME')+'/.eutester/'+str(fname)
        except: return None
        try:
            os.stat(def_path)
            return def_path
        except:
            self.debug("Default config not found:"+str(def_path))
            return None
    
    def show_self(self):
        list=[]
        list.append(("NAME:", str(self.name)))
        list.append(('TEST LIST:', str(self.testlist)))
        list.append(('CONFIG FILES:', self.configfiles))
        argbuf=""
        argbuf = str("TESTCASE INFO:").ljust(25)        
        argbuf += str("\n----------").ljust(25)
        for val in list:
            argbuf += '\n'+str(val[0]).ljust(25)+" --->:  "+str(val[1])
        self.status(argbuf)
        self.show_args()
        
        
        
        
        
    def show_args(self,args=None):
        '''
        Description: Prints args names and values for debug purposes. 
                     By default will use the local testcase.args, else args can be provided. 
        
        :type args: namespace object
        :param args: namespace object to be printed,by default None will print local testcase's args.
        '''
        args= args or self.args if hasattr(self,'args') else None
        argbuf = str("TEST ARGS:").ljust(25)+"        "+str("VALUE:")
        argbuf += str("\n----------").ljust(25)+"        "+str("------")
        if args:
            for val in args._get_kwargs():
                argbuf += '\n'+str(val[0]).ljust(25)+" --->:  "+str(val[1])
            self.status(argbuf)
            
    
    def populate_testunit_with_args(self,testunit,namespace=None):
        '''
        Description: Checks a given test unit's available positional and key word args lists for matching
                     values contained with the given namespace, by default will use local testcase.args. 
                     If testunit's underlying method has arguments matching the namespace provided, then those
                     args will be applied to the testunits args referenced when running the testunit. 
                     Namespace values will not be applied/overwrite testunits, if the testunit already has conflicting
                     values in it's args(positional) list or kwargs(keyword args) dict.
        :type: testunit: Eutestcase.eutestertestunit object
        :param: testunit: A testunit object for which the namespace values will be applied 
        
        :type: namespace: namespace obj
        :param: namespace: namespace obj containing args/values to be applied to testunit. None by default will use local
                            testunit args. 
        '''
        self.debug("Attempting to populate testunit:"+str(testunit.name)+", with testcase.args...")
        args_to_apply = namespace or self.args
        if not args_to_apply:
            return
        testunit_obj_args = {}
        
        #copy the test units key word args
        testunit_obj_args.update(copy.copy(testunit.kwargs))
        self.debug("Testunit keyword args:"+str(testunit_obj_args))
        
        #Get all the var names of the underlying method the testunit is wrapping
        method_args = self.get_meth_arg_names(testunit.method)
        offset = 0 if isinstance(testunit.method,types.FunctionType) else 1
        self.debug("Got method args:"+str(method_args))
       
            
        #Add the var names of the positional args provided in testunit.args to check against later
        #Append to the known keyword arg list
        for x,arg in enumerate(testunit.args):
            testunit_obj_args[method_args[x+offset]] = arg
        
        self.debug("test unit total args:"+str(testunit_obj_args))
        #populate any global args which do not conflict with args already contained within the test case
        #first populate matching method args with our global testcase args taking least precedence
        for apply_val in args_to_apply._get_kwargs():
            for methvar in method_args:
                if methvar == apply_val[0]:
                    self.debug("Found matching arg for:"+str(methvar))
                    #Don't overwrite existing testunit args/kwargs that have already been assigned
                    if apply_val[0] in testunit_obj_args:
                            self.debug("Skipping populate because testunit already has this arg:"+str(methvar))
                            continue
                    #Append cmdargs list to testunits kwargs
                    testunit.set_kwarg(methvar,apply_val[1]) 
                    #testunit.kwargs[methvar]=apply_val[1]
       
       
        
    
    def do_with_args(self, meth, *args, **kwargs):
        '''
        Description: Convenience method used to wrap the provided instance_method, function, or object type 'meth' and populate meth's positional 
        and keyword arguments with the local testcase.args created from the CLI and/or config file, as well as
        the *args and **kwargs variable length arguments passed into this method. 
        
        :type meth: method
        :param meth: A method or class initiator to wrapped/populated with this testcase objects namespace args
        
        :type args: positional arguments
        :param args: None or more values representing positional arguments to be passed to 'meth' when executed. These will
                     take precedence over local testcase obj namespace args
        
        :type kwargs: keyword arguments
        :param kwargs: None or more values reprsenting keyword arguments to be passed to 'meth' when executed. These will
                     take precedence over local testcase obj namespace args and positional args
        '''
        
        if not hasattr(self,'args'):
            raise Exception('TestCase object does not have args yet, see: get_args and setup_parser options')
        tc_args = self.args
        cmdargs={}
        f_code = self.get_method_fcode(meth)
        vars = self.get_meth_arg_names(meth)
        self.debug("do_with_args: Method:"+str(f_code.co_name)+", Vars:"+str(vars))
        
        #first populate matching method args with our global testcase args...
        for val in tc_args._get_kwargs():
            for var in vars:
                if var == val[0]:
                    cmdargs[var]=val[1]
        #Then overwrite/populate with any given positional local args...
        for count,arg in enumerate(args):
            cmdargs[vars[count+1]]=arg
        #Finall overwrite/populate with any given key word local args...
        for name,value in kwargs.items():
            for var in vars:
                if var == name:
                    cmdargs[var]=value
        self.debug('create_with_args: running '+str(f_code.co_name)+"("+str(cmdargs).replace(':','=')+")")
        return meth(**cmdargs)            
        
    @classmethod
    def get_method_fcode(cls, meth):
        f_code = None
        #Find the args for the method passed in...
        #Check for object/class init...
        if isinstance(meth,types.ObjectType):
            try:
                f_code = meth.__init__.__func__.func_code
            except:pass   
        #Check for instance method...
        if isinstance(meth,types.MethodType):
            try:
                f_code = meth.im_func.func_code
            except:pass    
        #Check for function...
        if isinstance(meth,types.FunctionType):
            try:
                f_code = meth.func_code
            except:pass
        if not f_code:
            raise Exception("get_method_fcode: Could not find function_code for passed method of type:"+str(type(meth)))
        return f_code
    
    @classmethod
    def get_meth_arg_names(cls,meth):
        '''
        Description: Return varnames within argcount        
        :type:meth: method
        :param: meth: method to fetch arg names for
        
        :rtype: list
        :returns: list of strings representing the varnames within argcount for this method
        '''
        fcode = cls.get_method_fcode(meth)
        varnames = fcode.co_varnames[0:fcode.co_argcount]
        return varnames



class SkipTestException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)
            
    
